#!/usr/bin/env python3

from fastapi import FastAPI, HTTPException, Request
from fastapi.middleware.cors import CORSMiddleware
from slowapi.errors import RateLimitExceeded
from slowapi import Limiter, _rate_limit_exceeded_handler
from slowapi.util import get_remote_address
import subprocess
import re

limiter = Limiter(key_func=get_remote_address)
app = FastAPI()
app.add_middleware(
    CORSMiddleware,
    allow_origins=["*"],
    allow_credentials=True,
    allow_methods=["*"],
    allow_headers=["*"],
)
app.state.limiter = limiter
app.add_exception_handler(RateLimitExceeded, _rate_limit_exceeded_handler)

ip_regex = re.compile(r"^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$")

@app.get("/api/rdns/{ip}")
@limiter.limit("1/second")
async def get_rdns(ip: str, request: Request):
    match = re.match(ip_regex, ip)
    if match is None:
        raise HTTPException(status_code=400, detail="Invalid IP address")
    sanitized_ip = match[0]
    output = subprocess.run(["dig", "-x", sanitized_ip, "+short"], capture_output = True, text = True).stdout.strip()
    if not output:
        raise HTTPException(status_code=404, detail="No rDNS information found for IP")
    return { "ip": sanitized_ip, "rdns": output.rstrip(".") }

if __name__ == "__main__":
    import uvicorn
    uvicorn.run(app, host="0.0.0.0", port=8000)
